==> parsec_cloud-2.15.0rc3-cp39-cp39-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: parsec/core/backend_connection/expose_cmds.py <==
# Parsec Cloud (https://parsec.cloud) Copyright (c) AGPL-3.0 2016-present Scille SAS
from __future__ import annotations

from functools import wraps
from typing import TYPE_CHECKING, Awaitable, Callable, TypeVar, Union

from typing_extensions import Concatenate, ParamSpec

from parsec.api.transport import Transport
from parsec.core.backend_connection.exceptions import BackendNotAvailable

if TYPE_CHECKING:
    from parsec.core.backend_connection.apiv1_anonymous import APIV1_BackendAnonymousCmds
    from parsec.core.backend_connection.authenticated import BackendAuthenticatedCmds
    from parsec.core.backend_connection.invited import BackendInvitedCmds

    T_BACKEND = Union[BackendInvitedCmds, BackendAuthenticatedCmds, APIV1_BackendAnonymousCmds]

P = ParamSpec("P")
R = TypeVar("R")


def expose_cmds(
    cmd: Callable[Concatenate[Transport, P], Awaitable[R]]
) -> Callable[Concatenate[T_BACKEND, P], Awaitable[R]]:
    @wraps(cmd)
    async def wrapper(self: T_BACKEND, *args: P.args, **kwargs: P.kwargs) -> R:
        async with self.acquire_transport() as transport:
            return await cmd(transport, *args, **kwargs)

    return wrapper


def expose_cmds_with_retrier(
    cmd: Callable[Concatenate[Transport, P], Awaitable[R]]
) -> Callable[Concatenate[T_BACKEND, P], Awaitable[R]]:
    @wraps(cmd)
    async def wrapper(self: T_BACKEND, *args: P.args, **kwargs: P.kwargs) -> R:
        # Reusing the transports expose us to `BackendNotAvailable` exceptions
        # due to inactivity timeout while the transport was in the pool.
        try:
            async with self.acquire_transport(allow_not_available=True) as transport:
                return await cmd(transport, *args, **kwargs)

        except BackendNotAvailable:
            async with self.acquire_transport(force_fresh=True) as transport:
                return await cmd(transport, *args, **kwargs)

    return wrapper

==> parsec_cloud-2.16.0rc4-cp39-cp39-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: parsec/core/backend_connection/expose_cmds.py <==
# Parsec Cloud (https://parsec.cloud) Copyright (c) AGPL-3.0 2016-present Scille SAS
from __future__ import annotations

from functools import wraps
from typing import TYPE_CHECKING, Awaitable, Callable, TypeVar, Union

from typing_extensions import Concatenate, ParamSpec

from parsec.api.transport import Transport
from parsec.core.backend_connection import BackendNotAvailable

if TYPE_CHECKING:
    from parsec.core.backend_connection.authenticated import BackendAuthenticatedCmds
    from parsec.core.backend_connection.invited import BackendInvitedCmds

    T_BACKEND = Union[BackendInvitedCmds, BackendAuthenticatedCmds]

P = ParamSpec("P")
R = TypeVar("R")


def expose_cmds(
    cmd: Callable[Concatenate[Transport, P], Awaitable[R]]
) -> Callable[Concatenate[T_BACKEND, P], Awaitable[R]]:
    @wraps(cmd)
    async def wrapper(self: T_BACKEND, *args: P.args, **kwargs: P.kwargs) -> R:
        async with self.acquire_transport() as transport:
            return await cmd(transport, *args, **kwargs)

    return wrapper


def expose_cmds_with_retrier(
    cmd: Callable[Concatenate[Transport, P], Awaitable[R]]
) -> Callable[Concatenate[T_BACKEND, P], Awaitable[R]]:
    @wraps(cmd)
    async def wrapper(self: T_BACKEND, *args: P.args, **kwargs: P.kwargs) -> R:
        # Reusing the transports expose us to `BackendNotAvailable` exceptions
        # due to inactivity timeout while the transport was in the pool.
        try:
            async with self.acquire_transport(allow_not_available=True) as transport:
                return await cmd(transport, *args, **kwargs)

        except BackendNotAvailable:
            async with self.acquire_transport(force_fresh=True) as transport:
                return await cmd(transport, *args, **kwargs)

    return wrapper
